local Ennemy = Class {

    IMG_Walk = lg.newImage(
        "images/walk.png"
    );

    init = function (self, x, y)

        self.model = Animation(self.IMG_Walk, .75)
        self.model:setScale(1.5,1.5,1)

        self.x, self.y, self.z = x, y, -.25

        self.model:setTranslation(
            self.x, self.y, self.z
        )

    end;

    update = function (self, dt)
        self.model:update(dt)
        self.model:setRotToCam(g3d.camera)
    end;

    draw = function (self, shader)
        self.model:draw(shader)
    end;

}


Ennemies = {}
Ennemies.__index = Ennemies

function Ennemies:reset()
    for i = #self, 1, -1 do
        table.remove(self, i)
    end
end

function Ennemies:setWorld(world)
    self.world = world
end

function Ennemies:add(x,y)
    self[#self+1] = Ennemy(x,y,self.world)
end

function Ennemies:update(dt)
    for i = #self, 1, -1 do
        if self[i]:update(dt) then
            table.remove(self, i)
        end
    end
end

local shader = lg.newShader(g3d.shaderpath, [[
    vec4 effect(vec4 color, Image tex, vec2 texCoords, vec2 screenCoords) {
        vec4 pixel = Texel(tex, texCoords);
        if(pixel.a < 0.5) discard;
        return pixel*color;
    }
]])

function Ennemies:draw()
    for _, ennemy in ipairs(self) do
        ennemy:draw(shader)
    end
end

return function ()
    return setmetatable({}, Ennemies)
end